#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['datapungi_fed', 'datapungi_fed.config', 'datapungi_fed.tests']

package_data = \
{'': ['*'], 'datapungi_fed': ['.vscode/*']}

install_requires = \
['requests',
 'pandas',
 'pytest',
 'pytest-html',
 'pyperclip',
 'lxml',
 'xlrd',
 'beautifulsoup4',
 'html5lib',
 'pyyaml']

setup(name='datapungi_fed',
      version='0.1.2',
      description='Gets data from Federal Reserve (FED) by connecting to its API.',
      author='James Otterson',
      author_email='jjotterson@gmail.com',
      url='https://github.com/jjotterson/datapungi_fed',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
     )
